#include "WinReg.h"

/**
 * deleteSubKey
 *
 * This function will remove a subkey from the windows registry
 *
 * @parameters
 * iRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *            HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * sRegKeyPath - The path to the key you want to delete
 * sRegSubKey - The name of the subkey you want to delete
 *
 * @example
 * deleteSubKey(HKEY_CURRENT_USER, "Software\Company\Stuff", "DataKey")
 */
void WinReg::deleteSubKey(HKEY hRootKey, const string &sRegKeyPath, const string &sRegSubKey)
{
	HKEY hKey;

	//Make sure the key exist before trying to delete it
	if (keyExist(hRootKey, sRegKeyPath.c_str()))
	{
		//Get the key handle
		lRetVal = RegOpenKey(hRootKey, sRegKeyPath.c_str(), &hKey);

		//Delete the sub key. If it doesn't exist we'll ignore it.
		lRetVal = RegDeleteValue(hKey, sRegSubKey.c_str());

		//Always close the handle in the registry.  We do not want to corrupt the registry
		lRetVal = RegCloseKey(hKey);
	}
}

/**
 * keyExist
 *
 * This function will check if a key exist
 *
 * @parameters
 * iRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *			  HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * sRegKeyPath - The path to the key you want to check
 *
 * @return value
 * Boolean, true if the key exist, false if it doesn't
 *
 * @example
 * keyExist(HKEY_CURRENT_USER, "Software\Company\Stuff")
 */
bool WinReg::keyExist(HKEY hRootKey, const string &sRegKeyPath)
{
	HKEY hKey = NULL;

	//Query the key path
	lRetVal = RegOpenKey(hRootKey, sRegKeyPath.c_str(), &hKey);

	//Always close the handle in the registry. We do not want to corrupt these files.
	lRetVal = RegCloseKey(hKey);

	return hKey != NULL;
}

/**
 * queryAKey
 *
 * This function will query (read) a sub key value
 *
 * @parameters
 * lngRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *              HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * strRegKeyPath - The path to the key you want to query
 * strRegSubKey - The name of the subkey you want to query
 *
 * @return value
 * Variant, we don't know what we'll be retrieving, but it'll be the value of the SubKey
 *
 * @example
 * regQueryAKey(HKEY_CURRENT_USER, "Software\Company\Stuff", "DataKey")
 */
string* WinReg::queryAKey(HKEY hRootKey, const string &sRegKeyPath, const string &sRegSubKey)
{
	HKEY hKey = NULL;
	DWORD dwDataType;
	DWORD dwSize;

	//Query the key path
	lRetVal = RegOpenKey(hRootKey, sRegKeyPath.c_str(), &hKey);

	//If no key handle was found then there is no key. Leave here.
	if (hKey == NULL)
	{
		lRetVal = RegCloseKey(hKey);   // always close the handle
		return new string("");
	}

	//Query the registry and determine the data type and the size of the data
	lRetVal = RegQueryValueEx(hKey, sRegSubKey.c_str(), NULL, &dwDataType, NULL, &dwSize);

	//If no key handle was found then there is no key. Leave.
	if (hKey == NULL)
	{
		lRetVal = RegCloseKey(hKey); // always close the handle
		return new string("");
	}

	//Allocate space for the data
	BYTE *bData = new BYTE[dwSize + 1];

	lRetVal = RegQueryValueEx(hKey, sRegSubKey.c_str(), NULL, NULL, bData, &dwSize);

	//If NOT a successful call then leave
	if (lRetVal != ERROR_SUCCESS)
	{
		lRetVal = RegCloseKey(hKey);
		return new string("");
	}

	string *sReturnVal = new string();

	//Make the API call to query the registry based on the type of data.
	switch (dwDataType)
	{
		case REG_SZ: //String data (most common)
		case REG_EXPAND_SZ:
		{
			//Convert the data to a string
			ostringstream oss;
			oss << bData;
			string sData = oss.str();

			delete [] bData;

			// Strip out the string data
			sData = sData.substr(0, sData.find_first_of('\0'));

			*sReturnVal = sData;

			break;
		}
		case REG_DWORD: //Numeric data (Integer)
		{
			DWORD dw;
			memcpy(&dw, bData, sizeof(DWORD));

			delete [] bData;

			//Save the captured data
			ostringstream oss;
			oss << dw;
			string sData = oss.str();

			// Strip out the string data
			sData = sData.substr(0, sData.find_first_of('\0'));

			*sReturnVal = sData;

			break;
		}
		default: //unknown
		{
			*sReturnVal = "";
			break;
		}
	}

	//Always close the handle in the registry. We do not want to corrupt these files.
	lRetVal = RegCloseKey(hKey);

	return sReturnVal;
}

/**
 * createKeyValue
 *
 * This function will save data to a key in the registry
 *
 * @parameters
 * hRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *            HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * sRegKeyPath - The path to the key you want to create
 * sRegSubKey - The name of the subkey you want to create
 * sRegData/iRegData - The data to write to the subkey
 *
 * @example
 * createKeyValue(HKEY_CURRENT_USER, "Software\Company\Stuff", "DataKey", "ActualData")
 */
void WinReg::createKeyValue(HKEY hRootKey, const string &sRegKeyPath, const string &sRegSubKey, const string &sRegData)
{
	HKEY hKey = NULL;

	//Query the key path
	lRetVal = RegCreateKey(hRootKey, sRegKeyPath.c_str(), &hKey);

	lRetVal = RegSetValueEx(hKey, sRegSubKey.c_str(), NULL, REG_SZ, reinterpret_cast<const BYTE *>(sRegData.c_str()), static_cast<DWORD>(sRegData.size()));

	//Always close the handle in the registry. We do not want to corrupt these files.
	lRetVal = RegCloseKey(hKey);
}

void WinReg::createKeyValue(HKEY hRootKey, const string &sRegKeyPath, const string &sRegSubKey, int iRegData)
{
	HKEY hKey = NULL;

	DWORD data = iRegData;

	//Query the key path
	lRetVal = RegCreateKey(hRootKey, sRegKeyPath.c_str(), &hKey);

	lRetVal = RegSetValueEx(hKey, sRegSubKey.c_str(), NULL, REG_DWORD, reinterpret_cast<const BYTE *>(&data), sizeof(data));

	//Always close the handle in the registry. We do not want to corrupt these files.
	lRetVal = RegCloseKey(hKey);
}

/**
 * createAKey
 *
 * This function will create a new key
 *
 * @parameters
 * hRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *            HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * sRegKeyPath - The path to the key you want to create. You should be able to
 *                create new keys to any level at once, but to be safe I would
 *                create the levels one at a time
 *
 * @example
 * createAKey(HKEY_CURRENT_USER, "Software\Company")
 * createAKey(HKEY_CURRENT_USER, "Software\Company\Stuff")
 */
void WinReg::createAKey(HKEY hRootKey, const string &sRegKeyPath)
{
	HKEY hKey = NULL;

	//Create the key. If it already exist, ignore it.
	lRetVal = RegCreateKey(hRootKey, sRegKeyPath.c_str(), &hKey);

	//Always close the handle in the registry. We do not want to corrupt these files.
	lRetVal = RegCloseKey(hKey);
}

/**
 * regDeleteAKey
 *
 * This function will remove a key and all it *s subkeys (might not on NT-systems)
 *
 * @parameters
 * hRootKey - HKEY_CLASSES_ROOT, HKEY_CURRENT_USER,
 *            HKEY_lOCAL_MACHINE, HKEY_USERS, etc
 * sRegKeyPath - The path to the key you want to delete
 * sRegKeyValue - The name of the key you want to delete
 *
 * @return value
 * Boolean, true if the key could be deleted, false if it couldn't
 *
 * @example
 * deleteAKey(HKEY_CURRENT_USER, "Software\Company", "Stuff")
 */
bool WinReg::deleteAKey(HKEY hRootKey, const string &sRegKeyPath, const string &sRegKeyName)
{
	HKEY hKey;
	bool bReturnValue = false; //Preset to a failed delete

	//Make sure the key exist before trying to delete it
	if (keyExist(hRootKey, sRegKeyPath))
	{
		//Get the key handle
		lRetVal = RegOpenKey(hRootKey, sRegKeyPath.c_str(), &hKey);

		//Delete the key
		lRetVal = RegDeleteKey(hKey, sRegKeyName.c_str());

		//If the value returned is equal zero then we have succeeded
		if (lRetVal == 0)
			bReturnValue = true;

		//Always close the handle in the registry. We do not want to corrupt the registry.
		lRetVal = RegCloseKey(hKey);
	}

	return bReturnValue;
}